#!/usr/bin/perl
# $Id: slis-sys.inc.pl,v 3.5 2003/07/11 09:02:57 bzizou Exp $ #
#
# Perl include for SLIS system subroutines
#
# This script is part of the SLIS Project initiated by the CARMI-Internet
# (Acadmie de Grenoble - France 38).
# Ce script fait partie du projet SLIS dmarr par le CARMI-Internet
# (Acadmie de Grenoble - France 38).
#
# SLIS : Serveur de communications Linux Pour l'Internet Scolaire.
# Copyright (C) 1998-2001 Bruno Bzeznik
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    You should have received a copy of the GNU General Public License
#    along with this program (For example ./COPYING);
#    if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
#    Cambridge, MA 02139, USA.
#
# Please send all comments and bug reports by electronic mail to:
#   Bruno Bzeznik <Bruno@ac-grenoble.fr>
# or to <slis@ac-grenoble.fr>
#
# Envoyez vos suggestions et reports de bugs par e-mail 
#   Bruno Bzeznik <Bruno@ac-grenoble.fr>
# ou  <slis@ac-grenoble.fr>
#

do '/home/hadmin/slis.conf.pl';
do '/home/hadmin/cgi-bin/base/config_pgsql.inc.pl';

$ALIASES="/etc/aliases";
$NEWALIASES="/usr/bin/newaliases";
$LANG_DIR="/home/hadmin";
$MKNTPWD="/usr/local/sbin/mkntpwd";

sub init_syslog { 	# Open a channel to syslog

  setlogsock('unix');
  openlog($_[0],'pid',$LOG_FACILITY);
  return 0;
}

sub load_language {	# Load the perl language file

  if (not defined($LANGUAGE)) {
    $LANGUAGE="french";
    syslog('warning',"WARN: Variable \$LANGUAGE not found in slis.conf.pl");
    syslog('warning',"WARN: using \"$LANGUAGE\" as the language for reports");
    $r=2;
  }
  
  if (-e "$LANG_DIR/$LANGUAGE.inc.pl") {do "$LANG_DIR/$LANGUAGE.inc.pl"; $r=0;}
  else {syslog('err',"ERR: File $LANG_DIR/$LANGUAGE.inc.pl not found!"); $r=1;}
  return $r;
}

sub sql_init {	# Init string to SQL server

  my $DRIVER="Pg";
  my $PORT="5432";
  my $dsn = "DBI:$DRIVER:dbname=$BASE_HADMIN;host=$PGSQL_SERVER;port=$PORT";
  return $dsn;
}

sub log_err { 		# Log the error of an SQL request: log_err($sth,$msg)

  my $str = $_[0]->errstr;
  if ($str ne "") { syslog('err',$_[1] .": ". $str) };
  return 0;    
}


sub log_msg { 		# Log an str to err: log_msg($msg)

  if ($_[0] ne "") { syslog('err',"ERR: ". $_[0]) };
  return 0;
}


sub clean_conns {	# Clean the "conns" table by deleting values older than one year: clean_conns($sth)

  $sth=$_[0];
  $sth = $dbh->prepare("delete from conns where date_part('days',now()-start)>365");
  $sth->execute;
  log_err($sth,'delete from conns');
}

sub sql_rotate {	# sql_rotate($sth,$table,$date_field,$number_of_days)
			# Clean a table by deleting values where date_field is older than number_of_days
  $sth=$_[0];
  $table=$_[1]; 
  $date=$_[2]; 
  $days=$_[3]; 
  $sth = $dbh->prepare("delete from $table where date_part('days',now()-$date)>$days");
  $sth->execute;
  log_err($sth,"delete from $table");
}



sub sql_tail {	        # sql_tail($sth,$table,$date_field,$number_of_lines)
			# Clean a table by deleting old values, keeping number_of_lines lines
  $sth=$_[0];
  $table=$_[1]; 
  $date=$_[2]; 
  $number_of_lines=$_[3]; 
  $sth = $dbh->prepare("create temp table \"access_tmp\" as select * from $table order by $date desc limit $number_of_lines");
  $sth->execute;
  log_err($sth,"create temp table access_tmp $table");
  $sth = $dbh->prepare("drop table $table");
  $sth->execute;
  log_err($sth,"drop table $table");
  $sth = $dbh->prepare("create table \"$table\" as select * from \"access_tmp\" order by $date");
  $sth->execute;
  log_err($sth,"create $table");
}


sub init_ldap {		# init_ldap()
			# Create the top OUs into the LDAP server

  syslog("notice","Initializing LDAP tree...");

  $SECRET=`cat /etc/openldap/secret`; chomp($SECRET);
  $LDAPADD="/usr/bin/ldapadd -D \"$LDAP_ROOT_DN\" -x -w $SECRET";

  $RND=rand 10000;

  ($TOP)=split(/,/,$BASE_DN);
  $TOP=~s/=/: /;
  ($OBJECT)=split(/:/,$TOP);
  if ($OBJECT eq "ou") { $OBJECTCLASS="objectclass: organizationalUnit"; }
  elsif ($OBJECT eq "o") { $OBJECTCLASS="objectclass: organization"; }
  elsif ($OBJECT eq "dc") { $OBJECTCLASS="objectclass: dcObject"; }
  else { syslog("warning","Use o,ou or dc for your top object in BASE_DN. Ldap server may not work...");}
  ($USERS_OU)=split(/,/,$LDAP_USERS);
  $USERS_OU=~s/=/: /;
  ($GROUPS_OU)=split(/,/,$LDAP_GROUPS);
  $GROUPS_OU=~s/=/: /;

  open (LDIF,">/tmp/initldap.$RND");
  print LDIF "
dn: $BASE_DN
objectclass: top
$OBJECTCLASS
$TOP

dn: $LDAP_USERS
objectclass: top
objectclass: organizationalUnit
$USERS_OU

dn: $LDAP_GROUPS
objectclass: top
objectclass: organizationalUnit
$GROUPS_OU
";
 
  close(LDIF);
  $ldapadd_err=`$LDAPADD -f /tmp/initldap.$RND 2>&1 1>/dev/null`;
  log_msg($ldapadd_err);

  unlink ("/tmp/initldap.$RND");

  syslog("notice","Initializing LDAP tree: Finished.");
}



sub useradd {	# useradd($comment,$pw,$shell,$home_dir,$attributes,$login)
		# add a user into LDAP server
		# $attributes may contain a list of attributes for this user, eg:
		# "employeenumber=1234,sn=Bruno,cn=Bruno Bzeznik"

  my $gid_file="/etc/openldap/slis_gid"; 	# A file containing the GID of users
  my $secret_file="/etc/openldap/secret"; 	# The manager passwd
  my $cpu="/usr/local/bin/cpu";		# The CPU utility (cpu.sourceforge.net)
  my $getent="/usr/bin/getent";		# The getent command
  my $ldapmodify="/usr/bin/ldapmodify";	# ldapmodify command

  my $comment=$_[0];
  my $pw=$_[1];
  if ($pw eq "md5" || $pw eq "crypt") {$pw="";}
  my $shell=$_[2];
  my $home_dir=$_[3];
  my $attributes=$_[4];
  $attributes =~ s/\n//;
  my $login=$_[5];
  open (GID,"$gid_file");
  my $gid=<GID>;
  close (GID);
  chomp($gid);
  open (SECRET,"$secret_file");
  my $secret=<SECRET>;
  close(SECRET);
  chomp($secret);
  my $error;

  if (not($gid =~ /^[0-9]+/)) {
    return "The GID is not found into $gid_file\n";
  }
  if ($login eq "" || pw eq "") {
    return "Can not create empty username or password\n";
  }
  elsif (`$getent passwd $login`) {
    return "User $login already exists\n";
  }
  else {
    $options="";
    if ($comment ne "") { $options .= " -c \"$comment\""; }
    if ($shell ne "") { $options .= " -s $shell"; }
    if ($home_dir ne "") { $options .= " -d $home_dir"; }
    syslog("notice","Adding user $login");    
    $error=`$cpu useradd -m -g $gid -p $pw $options $login 2>&1 >/dev/null`;
    if ($attributes =~ /.+=.+/) {
      $RND=rand 10000;
      open (TMP,">/tmp/ldapmodify.$RND");
      @list=split(/,/,$attributes);
      print TMP "dn: uid=$login,$LDAP_USERS\n";
      for my $elem (@list) {
        $line = $elem;
        $line =~ s/=/: /;
        if ($line =~ /%rid%/) {
          # Then, we must compute the Samba rid attribute from the Unix uid
          $uid=`id -u $login`;
          $rid=2*$uid+1000;
          $line =~ s/%rid%/$rid/;
        }
        print TMP $line ."\n";
      }
      close (TMP);
      $error_attributes=`$ldapmodify -D "$LDAP_ROOT_DN" -x -w $secret -f /tmp/ldapmodify.$RND 2>&1 1>/dev/null`;
      unlink ("/tmp/ldapmodify.$RND");
      log_msg($error_attributes);

    }
    return $error;
  }
}

sub remove_from_all_groups {	# remove_from_all_groups($login)
				# remove a user from all his groups

  my $cpu="/usr/local/bin/cpu";		# The CPU utility (cpu.sourceforge.net)
  my $login=$_[0];
  
  my $previous_groups=`id -n -G $login`;
  # my $primary_group=`id -g -n $login`; chomp ($primary_group);
  my $error;
  my $error_tmp;

  if ($previous_groups ne "") {
    @list=split(/ /,$previous_groups);
    for my $elem (@list) {
      chomp($elem);
      #if ($elem ne $primary_group) {
        $error_tmp = `$cpu groupdel -j $login $elem 2>&1 >/dev/null`;
        #if ($error_tmp ne "") { $error .= $error_tmp ."- user $login: group $elem -"; }
      #}
    }
  }
  if ($error ne "") { return "remove_from_all_groups failed: $error"; }
  else { return; }
}



sub usermod {	# useradd($comment,$pw,$shell,$home_dir,$attributes,$group,$groups,$login)
		# change a user into LDAP server
                # $group: the initial gid
                # $groups: blank spaced list of additional groups
		# $attributes may contain a list of attributes for this user, eg:
		# "employeenumber=1234,sn=Bruno,cn=Bruno Bzeznik"

  my $secret_file="/etc/openldap/secret"; 	# The manager passwd
  my $cpu="/usr/local/bin/cpu";		# The CPU utility (cpu.sourceforge.net)
  my $getent="/usr/bin/getent";		# The getent command
  my $ldapmodify="/usr/bin/ldapmodify";	# ldapmodify command

  my $comment=$_[0];
  my $pw=$_[1];
  if ($pw eq "md5" || $pw eq "crypt") {$pw="";}
  my $shell=$_[2];
  my $home_dir=$_[3];
  my $attributes=$_[4];
  $attributes =~ s/\n//;
  my $group=$_[5];
  my $groups=$_[6];
  my $login=$_[7];
  open (SECRET,"$secret_file");
  my $secret=<SECRET>;
  close(SECRET);
  chomp($secret);
  my $error;

  if ($login eq "") {
    return "Can not modify empty username\n";
  }
  elsif (not `$getent passwd $login`) {
    return "User $login not found\n";
  }
  else {
    
    if (not($group =~ /^[0-9]+/) && $group ne "") {
      # get the gid of this named group
      $group=`$getent group $group |cut -d: -f3`;
      chomp($group);
    }
    
    $options="";
    if ($comment ne "") { $options .= " -c \"$comment\""; }
    if ($shell ne "") { $options .= " -s $shell"; }
    if ($home_dir ne "") { $options .= " -d $home_dir"; }
    if ($group ne "") { $options .= " -g $group"; }
    if ($pw ne "") { $options .= " -p $pw"; }
    
    # Options changing
    if ($options ne "") {
      syslog("notice","Changing options of user $login"); 
      if ($DEBUG) { 
        print "DEBUG: $cpu usermod $options $login\n";
      }
      $error=`$cpu usermod $options $login 2>&1`;
    }
    if ($error ne "" ) { return "Usermod error: ".$error; }

    # Attributes changing
    if ($attributes =~ /.+=.+/) {
      syslog("notice","Changing attributes of user $login");    
      $RND=rand 10000;
      open (TMP,">/tmp/ldapmodify.$RND");
      @list=split(/,/,$attributes);
      print TMP "dn: uid=$login,$LDAP_USERS\n";
      for my $elem (@list) {
        $line = $elem;
        chomp($line);
        $line =~ s/=/: /;
        if ($line =~ /%rid%/) {
          # Then, we must compute the Samba rid attribute from the Unix uid
          $uid=`id -u $login`;
          $rid=2*$uid+1000;
          $line =~ s/%rid%/$rid/;
        }
        print TMP $line ."\n";
      }
      close (TMP);
      if ($DEBUG) {
        print "DEBUG: $ldapmodify -D \"$LDAP_ROOT_DN\" -x -w $secret -f /tmp/ldapmodify.$RND\n";
        $error_attributes=`$ldapmodify -D "$LDAP_ROOT_DN" -x -w $secret -f /tmp/ldapmodify.$RND`;
      }
      else { $error_attributes=`$ldapmodify -D "$LDAP_ROOT_DN" -x -w $secret -f /tmp/ldapmodify.$RND 2>&1 1>/dev/null`; }
      unlink ("/tmp/ldapmodify.$RND");
      log_msg($error_attributes);
    }
    if ($error ne "" ) { return "Ldapmodify error: ".$error; }

    # Groups changing
    if ($groups ne "") {
      syslog("notice","Changing additional groups of user $login");
      # Remove the user from all previous groups
      $error = remove_from_all_groups($login);
      if ($error ne "") { syslog('warning',"WARN: $error"); }
      $error="";
      # Add the user to groups
      #my $primary_group=`id -g -n $login`; chomp ($primary_group);
      @list=split(/ /,$groups);
      for my $elem (@list) {
        chomp($elem);
        #if ($elem ne $primary_group) {
          if ($DEBUG) {
            print "DEBUG: $cpu groupadd -j $login $elem\n";
          }
          $error_tmp = `$cpu groupadd -j $login $elem 2>&1`;
          if ($error_tmp ne "") { $error .= $error_tmp. "- user $login: group $elem -"; }
        #}
      }
    }
    if ($error ne "") {$error="groupadd failed: $error (probably, group not found into ldap server)"; }
    return $error
  }
}


sub groupadd {		# groupadd($group)
			# Creates a group

  my $cpu="/usr/local/bin/cpu";		# The CPU utility (cpu.sourceforge.net)
  my $timeout=5;			# Timeout, in seconds	
  my $error;

  my $group=$_[0];

  if ($group ne "") { 
    $error= `$cpu groupadd $group 2>&1`;
    # sleep 1; ### TO BE REMOVED WHEN CPU BUG WILL BE FIXED!! ###
    # if ($error eq "") {
      # Test for the effectively created group
      # my $ok=""; my $i=0;
      # while ($i<$timeout*4 && $ok eq "") {
        # #wait for 250 ms
        # select(undef, undef, undef, 0.25);
        # $i++;
        # $ok=`getent group $group`;
      # }
      # if ($i==$timeout*4) {
      #   return "groupadd failed: Timeout";
      # }
    # }
  }
  else {
    $error="Can not create an empty group";
  }
  return $error;
}

sub groupdel {		# groupdel($group)
			# Deletes a group

  my $cpu="/usr/local/bin/cpu";		# The CPU utility (cpu.sourceforge.net)
  my $error;

  my $group=$_[0];

  if ($group ne "") { 
    my $error= `$cpu groupdel -y $group 2>&1`;
  }  
  return $error;
}


sub userdel {		# userdel($user,$delete_home)
			# Deletes a user

  my $cpu="/usr/local/bin/cpu";		# The CPU utility (cpu.sourceforge.net)
  my $error;

  my $user=$_[0];
  my $option;

  if ($_[1] == 1) { $option = "-y -r"; }

  if ($user ne "") { 
    my $error= `$cpu userdel $option $user <<EOF
y
EOF 2>&1`;
    unlink ("/var/spool/mail/$user");
  }  
  return $error;
}


sub groupmod {		# groupmod($new_name,$old_name)

  my $cpu="/usr/local/bin/cpu";		# The CPU utility (cpu.sourceforge.net)
  my $error;

  my $new_name=$_[0];
  my $old_name=$_[1];

  if ($new_name ne "" && $old_name ne "" && $new_name ne $old_name) {
    $error= `$cpu groupmod -n $new_name $old_name 2>&1`;
  }  
  return $error;
}

sub ispasswd { 		# Test if a string from the database is a password or not: ispasswd($string)

  if ($_[0] ne "md5" && $_[0] ne "crypt" && $_[0] ne "") { return 1; }
  else { return 0; };
}

sub get_setup_variable {# get_setup_variable($variable)
			# Get a variable from the setup file (setup.data)
  $setup_file="/home/hadmin/setup.data";
  if ($_[0] ne "") {
    if (not open(SETUP,$setup_file)) {
      syslog('err',"Can not open $setup_file");
      return "";
    }
    else {
      while (<SETUP>) {
        ($var,$remain)=split(/=/,$_,2);
        if ($var eq $_[0]) { 
          ($value,$comment)=split(/#/,$remain,2);
          $value=~s/\"//g;
          $value=~s/ //g;
          $value=~s/	//g;
          chomp($value);
          return $value; } 
      }
    }
  }
  else {return "";}
}

sub get_option {	# get_option($name)
			# get the value of an option from the options table
  $name=$_[0];
  $sth = $dbh->prepare("select value from options where name='$name'");
  $sth->execute;
  log_err($sth,"select from options");
  if ($sth->errstr eq "") { return $sth->fetchrow; }
  else { return "";}
}
